/*
 * Copyright (C) 1995 Phil Servita (meister@ftp.com)
 */	  
/*
 * Windows OTP One-Time Passphrase generator
 * Currently supports MD4, MD5, and SHA as hash algorithms.
 *
 * Edit History:
 *
 *     2-May-95		phil servita (meister@ftp.com)  Created.
 * to  5-May-95	             added hex display, decrement button, 
 *                           cr/lf append to clipboard.	 
 *     4-Jun-95		phil servita - added SHA
 */

#include <windows.h>
#include <stdio.h>
#include <string.h>

#include "winotp.h"
#include "sizes.h"

#include "otp.h"
/* #include "md4.h" */
/* #include "md5.h" */

extern BOOL FAR PASCAL AboutDlgProc (HWND, WORD, WORD, LONG);
extern BOOL FAR PASCAL WinOTPDlgProc (HWND, WORD, WORD, LONG);

long FAR PASCAL WndProc (HWND, WORD, WORD, LONG);
FARPROC  lpfnAboutDlgProc = NULL;
HANDLE   hAboutDlgInstance    = NULL;
FARPROC  lpfnWinOTPDlgProc = NULL;
HANDLE   hWinOTPDlgInstance    = NULL;

char *ComputePassPhrase (char *params, char *passwd);

void SetHashAlg (char *);
void SetClipAppendStr (char *);

int CopyPassphraseToClipBoard (char *pass);
char *GetClipboardContents (void);

void DisplayAsWords (void);
void DisplayAsHex (void);

void BoxAndBeep (HWND hwnd, char *msg, char *caption, int mode);

HWND   MAIN_WIND;
HANDLE MAIN_INSTANCE;

static char HashAlgUsed   [32];
static char ClipAppendStr [8];
static int  DisplayHex = 0;

char        dbgmsg [128];

int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance, 
                    LPSTR lpszCmdParam, int nCmdShow)
{
    static char szAppName [] = "OTPGen";
    HWND        hwnd;
    MSG         msg;
    WNDCLASS    wndclass;
    WNDCLASS	dlgwndclass;
    
    MAIN_INSTANCE = hInstance;

    if (!hPrevInstance)
    {   wndclass.style           = CS_HREDRAW | CS_VREDRAW | WS_CAPTION;
        wndclass.lpfnWndProc     = (WNDPROC) WndProc;
        wndclass.cbClsExtra      = NULL;
        wndclass.hInstance       = hInstance;
        wndclass.hIcon           = LoadIcon (hInstance, "OtpIcon"); 
        wndclass.hCursor         = LoadCursor (NULL, IDC_ARROW);
        wndclass.hbrBackground   = GetStockObject (WHITE_BRUSH);
        wndclass.lpszMenuName    = NULL;
        wndclass.lpszClassName   = szAppName;
        RegisterClass (&wndclass);

	dlgwndclass.style           = CS_HREDRAW | CS_VREDRAW | WS_CAPTION;
        dlgwndclass.lpfnWndProc     = (WNDPROC) DefDlgProc;
        dlgwndclass.cbClsExtra      = DLGWINDOWEXTRA;
        dlgwndclass.hInstance       = hInstance;
        dlgwndclass.hIcon           = LoadIcon (hInstance, "OtpIcon"); 
        dlgwndclass.hCursor         = LoadCursor (NULL, IDC_ARROW);
        dlgwndclass.hbrBackground   = GetStockObject (WHITE_BRUSH);
        dlgwndclass.lpszMenuName    = NULL;
        dlgwndclass.lpszClassName   = "WINOTPDLGCLASS";
        RegisterClass (&dlgwndclass);
	
    }

    hwnd = CreateWindow (szAppName, 
                         "OTP Generator",     /* class name */
                         WS_OVERLAPPEDWINDOW,      /* window style */
                         CW_USEDEFAULT,            /* initial X pos */
                         CW_USEDEFAULT,            /* initial Y pos */
                         CW_USEDEFAULT,            /* initial x size */
                         CW_USEDEFAULT,            /* initial y size */
                         NULL,                     /* parent window */
                         NULL,                     /* window menu */
                         hInstance,                /* program ihandle */
                         NULL                      /* creation params */
			);

    MAIN_WIND = hwnd;		
    SetHashAlg ("MD5");

/*  ShowWindow   (hwnd, nCmdShow); */
/*  UpdateWindow (hwnd);           */

    while (GetMessage (&msg, NULL, 0, 0))
    {      TranslateMessage (&msg);
           DispatchMessage (&msg);
    }

    return msg.wParam;
}

short cxChar, cyChar;

long FAR PASCAL WndProc (HWND hwnd, WORD message, WORD wParam, LONG lParam)
{
    HDC             hdc;
    PAINTSTRUCT     ps;
    RECT            rect;
    TEXTMETRIC      txtmetric;
    
    switch (message)
    {       
	    case WM_CREATE:
	         hdc = GetDC (hwnd);
	         GetTextMetrics (hdc, &txtmetric);
                 cxChar = txtmetric.tmAveCharWidth;
                 cyChar = txtmetric.tmHeight + txtmetric.tmExternalLeading;
                 ReleaseDC (hwnd, hdc);
                 // set initial window width & height to 70x12 chars
                 GetWindowRect (hwnd, &rect);
                 MoveWindow (hwnd, rect.left, rect.top,
                             (70 * cxChar),
                             (12 * cyChar + GetSystemMetrics (SM_CYCAPTION)),
                             FALSE);
		 
		 hAboutDlgInstance = ((LPCREATESTRUCT) lParam)->hInstance;
		 lpfnAboutDlgProc = MakeProcInstance (AboutDlgProc, 
                                                        hAboutDlgInstance);

                 hWinOTPDlgInstance = ((LPCREATESTRUCT) lParam)->hInstance;
		 lpfnWinOTPDlgProc = MakeProcInstance (WinOTPDlgProc, 
                                                        hWinOTPDlgInstance);

                 (void) DialogBox (hWinOTPDlgInstance, "WinOTPDlg",
                                   hwnd, lpfnWinOTPDlgProc);

                 PostQuitMessage (0);
                 return 0;
			 
            case WM_PAINT:
                 hdc = BeginPaint (hwnd, &ps);
		 SelectObject (hdc, GetStockObject (SYSTEM_FIXED_FONT));
                 GetClientRect (hwnd, &rect);
                 EndPaint (hwnd, &ps);
                 return 0;
		 
            case WM_SETFOCUS:
                 return 0;

            case WM_COMMAND:
		 switch (wParam)
                 {       
                         default:
		              return 0;
                 }

            case WM_DESTROY:
                 PostQuitMessage (0);
                 return 0;

    }
    return DefWindowProc (hwnd, message, wParam, lParam);
}

/* 
 * beep and display a standard message box.
 */

void BoxAndBeep (HWND hwnd, char *msg, char *caption, int mode)
{
     MessageBeep (0);
     MessageBox (hwnd, (LPSTR) msg, (LPSTR) caption, mode);
}

char *ComputePassPhrase (char *params, char *passwd)
{
	int n;
	static char otpasswd [256];
        char seed [128];
	char key[8];

        OTPkeycrunchfcn keycrunch_fcn;
        OTPiteratefcn   iterate_fcn;

        /*
         * Based on the algorithm specified, get the right keycrunch 
         * and iterate functions.
         */

	keycrunch_fcn = OTP_GetKeyCrunchFcn (HashAlgUsed);
        if (!keycrunch_fcn)
        {   return (char *) 0;
        }
	iterate_fcn = OTP_GetIterateFcn (HashAlgUsed);
        if (!iterate_fcn)
        {   return (char *) 0;
        }

	if (sscanf (params, "%d %s", &n, seed) != 2)
        {   return (char *) 0;
        }

	/* Crunch seed and password into starting key */
	if (OTP_KeyCrunch (key, seed, passwd, keycrunch_fcn) != 0)
        {   return (char *) 0;
	}

        while (n-- != 0)
               OTP_IterateHash (key, iterate_fcn);

        if (DisplayHex)
            (void) OTP_put8 (otpasswd, key);		
        else
            (void) OTP_btoe (otpasswd, key);

	return otpasswd;
}

/*
 * stuff the current one-time passphrase into the clipboard. 
 * append to it the current append string.
 */
	 
int CopyPassphraseToClipBoard (char *pass)
{
    int plen, appendlen, i;

    HANDLE hGlobal;
    LPSTR sptr;

    plen = strlen (pass);
    appendlen = strlen (ClipAppendStr);

    hGlobal = GlobalAlloc (GHND, (DWORD) plen + appendlen + 1);
    if (!hGlobal)
        return 0;

    sptr = GlobalLock (hGlobal);
    for (i = 0 ; i < plen ; i++)
    {    *sptr++ = pass [i];
    }
    for (i = 0 ; i < appendlen ; i++)
    {    *sptr++ = ClipAppendStr [i];
    }

    GlobalUnlock (hGlobal);
    OpenClipboard (MAIN_WIND);
    EmptyClipboard ();
    SetClipboardData (CF_TEXT, hGlobal);
    CloseClipboard ();

    return 1;
}

/* 
 * return the first 127 chars of the clipboard contents, if there 
 * is text available. returns a pointer to static data! Copy before
 * using again!
 */

char *GetClipboardContents ()
{   static char retval [128];
    HANDLE clipmem;
    LPSTR  cptr;

    if (!IsClipboardFormatAvailable (CF_TEXT))
        return (char *) 0;
    if (!OpenClipboard (MAIN_WIND))
        return (char *) 0;

    clipmem = GetClipboardData (CF_TEXT);
    cptr = GlobalLock (clipmem);

    strncpy (retval, cptr, 127);
    retval [127] = '\0';
    GlobalUnlock (clipmem);
    CloseClipboard ();

    return retval;

}    

void SetHashAlg (char *alg)
{   char wintitle [64];
    sprintf (wintitle, "OTP One-Time Passphrase Generator [%s]", alg);    
    SetWindowText (MAIN_WIND, (LPSTR) wintitle);    
    strcpy (HashAlgUsed, alg);
}

void SetClipAppendStr (char *str)
{   
    strncpy (ClipAppendStr, str, sizeof (ClipAppendStr) - 1);
    ClipAppendStr [sizeof (ClipAppendStr) - 1] = '\0';
}

void DisplayAsHex ()
{
    DisplayHex = 1;
}

void DisplayAsWords ()
{
    DisplayHex = 0;
}
